@@ -31,4 +31,9 @@ class RoadsAndLanesNotAdjusted(Exception):
 class MixOfGeometryAddition(Exception):
     """ Raise when the wrong way of adding geometires are used for the planview
     """
+    pass
+
+class GeneralIssueInputArguments(Exception):
+    """ Raise when there is something wrong in the input arguments 
+    """
     pass
\ No newline at end of file
