@@ -7,18 +7,21 @@
     - Creation of the junction based on the connecting roads and incomming/outgoing roads
 """
 import numpy as np
+import pyclothoids as pcloth
 
 from .lane import Lane, RoadMark, LaneSection, Lanes
 from .enumerations import RoadMarkType, MarkRule, ContactPoint, ElementType, ObjectType
 
-from .geometry import Line, Arc, Spiral, EulerSpiral, PlanView
+from .geometry import Line, Arc, Spiral, PlanView
 from .opendrive import Road, OpenDrive
 from .links import Junction, Connection, _get_related_lanesection, LaneLinker
+from .exceptions import GeneralIssueInputArguments
 
 
 STD_ROADMARK_SOLID = RoadMark(RoadMarkType.solid,0.2)
 STD_ROADMARK_BROKEN = RoadMark(RoadMarkType.broken,0.2)
 STD_START_CLOTH = 1/1000000000
+
 def standard_lane(offset=3,rm = STD_ROADMARK_BROKEN):
     """ standard_lane creates a simple lane with an offset an a roadmark
         
@@ -264,7 +267,6 @@ def create_straight_road(road_id, length=100, junction=-1, n_lanes=1, lane_offse
     # finally create the roads 
     return Road(road_id,planview1,lanes1,road_type=junction)
 
-
 def create_cloth_arc_cloth(arc_curv, arc_angle, cloth_angle, r_id, junction=1, cloth_start=STD_START_CLOTH, n_lanes=1, lane_offset=3):
     """ creates a curved Road  with a Spiral - Arc - Spiral, and two lanes
 
@@ -305,7 +307,7 @@ def create_cloth_arc_cloth(arc_curv, arc_angle, cloth_angle, r_id, junction=1, c
     
     # create geometries
     spiral1 = Spiral(cloth_start, arc_curv, angle=cloth_angle)
-    arc = Arc(arc_curv, angle=arc_angle )
+    arc = Arc(arc_curv, angle=arc_angle)
     spiral2 = Spiral(arc_curv, cloth_start, angle=cloth_angle)
 
     pv.add_geometry(spiral1)
@@ -323,6 +325,69 @@ def create_cloth_arc_cloth(arc_curv, arc_angle, cloth_angle, r_id, junction=1, c
     # create road
     return Road(r_id,pv,lanes,road_type=junction)
 
+def create_3cloths(cloth1_start, cloth1_end, cloth1_length, cloth2_start, cloth2_end, cloth2_length, cloth3_start, cloth3_end, cloth3_length, r_id, junction=1, n_lanes=1, lane_offset=3):
+    """ creates a curved Road  with a Spiral - Arc - Spiral, and two lanes
+
+        Parameters
+        ----------
+            cloth1_start (float): initial curvature of spiral 1 
+
+            cloth1_end (float): ending curvature of spiral 1 
+
+            cloth1_length (float): total length of spiral 1 
+
+            cloth2_start (float): initial curvature of spiral 2
+
+            cloth2_end (float): ending curvature of spiral 2 
+
+            cloth2_length (float): total length of spiral 2
+
+            cloth3_start (float): initial curvature of spiral 3 
+
+            cloth3_end (float): ending curvature of spiral 3 
+
+            cloth3_length (float): total length of spiral 3
+
+            r_id (int): the id of the road
+
+            junction (int): if the Road belongs to a junction
+                default: 1
+
+            cloth_start (float): staring curvature of clothoids
+
+            n_lanes (int): number of lanes
+                default: 1
+
+            lane_offset (int): width of the road
+                default: 3
+
+        Returns
+        -------
+            road (Road): a road built up of a Spiral-Spiral-Spiral
+    """
+    
+    pv = PlanView()
+    
+    # create geometries
+    spiral1 = Spiral(cloth1_start, cloth1_end, length=cloth1_length)
+    spiral2 = Spiral(cloth2_start, cloth2_end, length=cloth2_length)
+    spiral3 = Spiral(cloth3_start, cloth3_end, length=cloth3_length)
+
+    pv.add_geometry(spiral1)
+    pv.add_geometry(spiral2)
+    pv.add_geometry(spiral3)
+
+    # create lanes
+    lsec = LaneSection(0,standard_lane())
+    for i in range(1, n_lanes+1, 1):
+        lsec.add_right_lane(standard_lane(lane_offset))
+        lsec.add_left_lane(standard_lane(lane_offset))
+    lanes = Lanes()
+    lanes.add_lanesection(lsec)
+
+    # create road
+    return Road(r_id,pv,lanes,road_type=junction)
+
 def get_lanes_offset(road1, road2, contactpoint):
     """ returns number of lanes (hp #left lanes = # right lanes) and their offset (hp offset is constant)
 
@@ -354,8 +419,6 @@ def get_lanes_offset(road1, road2, contactpoint):
     return n_lanes, lane_offset 
 
 
-
-
 def create_junction_roads_standalone(angles,r,junction=1,spiral_part = 1/3, arc_part = 1/3,startnum=100,n_lanes=1,lane_width=3):
     """ creates all needed roads for some simple junctions, the curved parts of the junction are created as a spiral-arc-spiral combo 
         - 3way crossings (either a T junction, or 120 deg junction)
@@ -394,11 +457,10 @@ def create_junction_roads_standalone(angles,r,junction=1,spiral_part = 1/3, arc_
     angle_cloth = angle*spiral_part 
     spiral_length = 2*abs(angle_cloth*r)
 
-    spiral = EulerSpiral.createFromLengthAndCurvature(spiral_length, STD_START_CLOTH, 1/r)
-    (X, Y, _) = spiral.calc(spiral_length, 0, 0, STD_START_CLOTH, 0)
+    cloth = pcloth.Clothoid.StandardParams(0, 0, 0, STD_START_CLOTH, (1/r - STD_START_CLOTH) / spiral_length, spiral_length)
 
-    X0 = X-r*np.sin(angle_cloth)
-    Y0 = Y-r*(1-np.cos(angle_cloth))
+    X0 = cloth.XEnd-r*np.sin(angle_cloth)
+    Y0 = cloth.YEnd-r*(1-np.cos(angle_cloth))
     linelength = 2*(X0 + r + Y0)
 
     junction_roads = []
@@ -428,8 +490,8 @@ def create_junction_roads_standalone(angles,r,junction=1,spiral_part = 1/3, arc_
             junction_roads.append(tmp_junc)
 
     return junction_roads
-    
-def create_junction_roads(roads,angles,r,junction=1,spiral_part = 1/3, arc_part = 1/3,startnum=100):
+
+def create_junction_roads_from_arc(roads,angles,r=0,junction=1,arc_part = 1/3,startnum=100):
     """ creates all needed roads for some simple junctions, the curved parts of the junction are created as a spiral-arc-spiral combo
         Supported junctions:
         - 3way crossings (either a T junction, or 120 deg junction)
@@ -440,7 +502,7 @@ def create_junction_roads(roads,angles,r,junction=1,spiral_part = 1/3, arc_part
             roads (list of Road): all roads that should go into the junction
 
             angles (list of float): the angles from where the roads should be coming in (see description for what is supported), 
-                                    to be defined in mathimatically positive order, beginning with the first incoming road
+                                    to be defined in mathimatically positive order, beginning with the first incoming road [0, +2pi]
 
             r (float): the radius of the arcs in the junction (will determine the size of the junction)
             
@@ -460,18 +522,18 @@ def create_junction_roads(roads,angles,r,junction=1,spiral_part = 1/3, arc_part
             junction_roads (list of Road): a list of all roads needed for all traffic connecting the roads
     """
 
-    # if a straight line is used, calculate the length of it. Some Spiral Magic going on...
-    # http://www.jerrymahun.com/index.php/home/open-access/viii-curves/76-chapter-e-spirals?showall=1
+    #arc_part = 1 - 2*spiral_part
+    spiral_part = (1 - arc_part)/2 
 
     angle = np.pi/2
     angle_cloth = angle*spiral_part 
     spiral_length = 2*abs(angle_cloth*r)
+    
+    cloth = pcloth.Clothoid.StandardParams(0, 0, 0, STD_START_CLOTH, (1/r - STD_START_CLOTH) / spiral_length, spiral_length)
 
-    spiral = EulerSpiral.createFromLengthAndCurvature(spiral_length, STD_START_CLOTH, 1/r)
-    (X, Y, _) = spiral.calc(spiral_length, 0, 0, STD_START_CLOTH, 0)
-
-    X0 = X-r*np.sin(angle_cloth)
-    Y0 = Y-r*(1-np.cos(angle_cloth))
+    X0 = cloth.XEnd-r*np.sin(angle_cloth)
+    Y0 = cloth.YEnd-r*(1-np.cos(angle_cloth))
+    
     linelength = 2*(X0 + r + Y0)
 
     junction_roads = []
@@ -487,23 +549,115 @@ def create_junction_roads(roads,angles,r,junction=1,spiral_part = 1/3, arc_part
             roads[i].add_predecessor(ElementType.junction,junction)
         
         for j in range(1+i,len(roads)):
-            # check angle needed for junction
-            an = np.sign(angles[j]-angles[i]-np.pi)
+            # check angle needed for junction [-pi, +pi]
             an1 = angles[j]-angles[i] -np.pi
-            angle_arc = an1*arc_part
-
-            angle_cloth = an1*spiral_part
-
             #adjust angle if multiple of pi
             if an1 > np.pi: 
                 an1 = -(2*np.pi - an1)
 
+            angle_arc = an1*arc_part
+            angle_cloth = an1*spiral_part
+
+            sig = np.sign(an1)
+
             # create road, either straight or curved
             n_lanes, lanes_offset = get_lanes_offset(roads[i], roads[j], cp )
-            if an == 0:
+            if sig == 0:
+                # create straight road 
                 tmp_junc = create_straight_road(startnum,length= linelength,junction=junction, n_lanes=n_lanes, lane_offset=lanes_offset)
             else: 
+                # create the cloth-arc-cloth road given the radius fo the arc
                 tmp_junc = create_cloth_arc_cloth(  1/r , angle_arc , angle_cloth , startnum , junction, n_lanes=n_lanes, lane_offset=lanes_offset )
+                
+            # add predecessor and successor
+            tmp_junc.add_predecessor(ElementType.road,roads[i].id,cp)
+            tmp_junc.add_successor(ElementType.road,roads[j].id,ContactPoint.start)
+            startnum += 1
+            junction_roads.append(tmp_junc)
+
+    # add junction to the last road aswell since it's not part of the loop
+    roads[-1].add_predecessor(ElementType.junction,junction)
+
+    return junction_roads
+
+
+def create_junction_roads(roads , angles, R, junction=1, arc_part = 1/3,startnum=100):
+    """ creates all needed roads for some simple junctions, the curved parts of the junction are created as a spiral-arc-spiral combo
+        R is value to the the radius of the whole junction (meaning R = distance between the center of the junction and any external road attached to the junction)
+        Supportes all angles and number of roads.
+
+        Parameters
+        ----------
+            roads (list of Road): all roads that should go into the junction
+
+            angles (list of float): the angles from where the roads should be coming in (see description for what is supported), 
+                                    to be defined in mathimatically positive order, beginning with the first incoming road [0, +2pi]
+
+            R (list of float): the radius of the whole junction, meaning the distance between roads and the center of the junction. If only one value is specified, then all roads will have the same distance.
+            
+            junction (int): the id of the junction
+                default: 1
+
+            spiral_part (float): the part of the curve that should be spirals (two of these) spiral_part*2 + arcpart = angle of the turn
+                default: (1/3)
+
+            arc_part (float): the part of the curve that should be an arc:  spiral_part*2 + arcpart = angle of the turn
+                default: (1/3)
+
+            startnum (int): start number of the roads in the junctions (will increase with 1 for each road)
+
+        Returns
+        -------
+            junction_roads (list of Road): a list of all roads needed for all traffic connecting the roads
+    """
+
+
+    if (len(roads) is not len(angles)): 
+        raise GeneralIssueInputArguments('roads and angles do not have the same size.')
+
+    if len(R) == 1: 
+        R = R*np.ones(len(roads))
+    elif len(R) > 1 and len(R) is not len(roads): 
+        raise GeneralIssueInputArguments('roads and R do not have the same size.')
+
+
+    #arc_part = 1 - 2*spiral_part
+    spiral_part = (1 - arc_part)/2
+
+    #linelength = 2*R
+    junction_roads = []
+
+    # loop over the roads to get all possible combinations of connecting roads
+    for i in range(len(roads)-1):
+        # for now the first road is place as base, 
+        if i == 0:
+            cp = ContactPoint.end
+            roads[i].add_successor(ElementType.junction,junction)
+        else:
+            cp = ContactPoint.start
+            roads[i].add_predecessor(ElementType.junction,junction)
+        
+        for j in range(1+i,len(roads)):
+            # check angle needed for junction [-pi, +pi]
+            an1 = angles[j]-angles[i] -np.pi
+            #adjust angle if multiple of pi
+            if an1 > np.pi: 
+                an1 = -(2*np.pi - an1)
+
+            angle_arc = an1*arc_part
+            angle_cloth = an1*spiral_part
+            sig = np.sign(an1)
+
+
+            # create road, either straight or curved
+            n_lanes, lanes_offset = get_lanes_offset(roads[i], roads[j], cp )
+            if sig == 0:
+                # create straight road 
+                linelength = R[i]+R[j]
+                tmp_junc = create_straight_road(startnum,length= linelength,junction=junction, n_lanes=n_lanes, lane_offset=lanes_offset)
+            else: 
+                clothoids = pcloth.SolveG2(-R[i], 0, 0, STD_START_CLOTH, R[j]*np.cos(an1), R[j]*np.sin(an1), an1, STD_START_CLOTH)
+                tmp_junc = create_3cloths(clothoids[0].KappaStart, clothoids[0].KappaEnd, clothoids[0].length, clothoids[1].KappaStart, clothoids[1].KappaEnd, clothoids[1].length, clothoids[2].KappaStart, clothoids[2].KappaEnd, clothoids[2].length, startnum, junction, n_lanes=n_lanes, lane_offset=lanes_offset)
 
             # add predecessor and successor
             tmp_junc.add_predecessor(ElementType.road,roads[i].id,cp)
